@@ -1,16 +1,30 @@
 #!/usr/bin/env python
 
+from __future__ import annotations
+
 import argparse
+import signal
 import sys
 
-from grascii import config, dictionary, search
+from grascii import APP_NAME, __version__, config, dictionary, search
 
 
 def main() -> None:
-    argparser = argparse.ArgumentParser(prog="grascii")
+    try:
+        from pytest_cov.embed import cleanup_on_signal
+    except ImportError:
+        pass
+    else:
+        cleanup_on_signal(signal.SIGHUP)
+
+    argparser = argparse.ArgumentParser(prog=APP_NAME)
     subparsers = argparser.add_subparsers(title="subcommands")
     argparser.set_defaults(func=None)
 
+    argparser.add_argument(
+        "-V", "--version", action="version", version=f"%(prog)s {__version__}"
+    )
+
     search_parser = subparsers.add_parser(
         "search", description=search.description, help=search.description, aliases=["s"]
     )
