@@ -1,10 +1,12 @@
+from __future__ import annotations
+
 import os
 import unittest
 
 from lark import Lark
 from lark.visitors import CollapseAmbiguities
 
-from grascii.dephrase import PhraseFlattener
+from grascii.dephrase import PhraseFlattener, dephrase
 from grascii.grammars import get_grammar
 
 
@@ -109,5 +111,10 @@ class TestLessonPhrases(unittest.TestCase):
         self._test_lesson("19a")
 
 
+def test_aggressive():
+    dephrase(phrase="thl-nbg", aggressive=True)
+    dephrase(phrase="O(,^TH,TM", aggressive=True)
+
+
 if __name__ == "__main__":
     unittest.main()
